@@ -184,7 +184,13 @@ static int tlshd_server_x509_verify_function(gnutls_session_t session)
 		gnutls_x509_crt_t cert;
 
 		gnutls_x509_crt_init(&cert);
-		gnutls_x509_crt_import(cert, &peercerts[i], GNUTLS_X509_FMT_DER);
+		ret = gnutls_x509_crt_import(cert, &peercerts[i],
+					     GNUTLS_X509_FMT_DER);
+		if (ret != GNUTLS_E_SUCCESS) {
+			tlshd_log_gnutls_error(ret);
+			gnutls_x509_crt_deinit(cert);
+			return GNUTLS_E_CERTIFICATE_ERROR;
+		}
 		parms->remote_peerid[i] =
 			tlshd_keyring_create_cert(cert, parms->peername);
 		gnutls_x509_crt_deinit(cert);
@@ -201,6 +207,7 @@ static void tlshd_server_x509_handshake(struct tlshd_handshake_parms *parms)
 {
 	gnutls_certificate_credentials_t xcred;
 	gnutls_session_t session;
+	char *cafile;
 	int ret;
 
 	ret = gnutls_certificate_allocate_credentials(&xcred);
@@ -209,7 +216,12 @@ static void tlshd_server_x509_handshake(struct tlshd_handshake_parms *parms)
 		return;
 	}
 
-	ret = gnutls_certificate_set_x509_system_trust(xcred);
+	if (tlshd_config_get_server_truststore(&cafile)) {
+		ret = gnutls_certificate_set_x509_trust_file(xcred, cafile,
+							     GNUTLS_X509_FMT_PEM);
+		free(cafile);
+	} else
+		ret = gnutls_certificate_set_x509_system_trust(xcred);
 	if (ret < 0) {
 		tlshd_log_gnutls_error(ret);
 		goto out_free_creds;
@@ -233,7 +245,11 @@ static void tlshd_server_x509_handshake(struct tlshd_handshake_parms *parms)
 	gnutls_transport_set_int(session, parms->sockfd);
 	gnutls_session_set_ptr(session, parms);
 
-	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
+	ret = gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
+	if (ret != GNUTLS_E_SUCCESS) {
+		tlshd_log_gnutls_error(ret);
+		goto out_free_creds;
+	}
 	gnutls_certificate_set_verify_function(xcred,
 					       tlshd_server_x509_verify_function);
 	gnutls_certificate_server_set_request(session, GNUTLS_CERT_REQUEST);
@@ -339,7 +355,10 @@ void tlshd_serverhello_handshake(struct tlshd_handshake_parms *parms)
 	gnutls_global_set_log_function(tlshd_gnutls_log_func);
 	gnutls_global_set_audit_log_function(tlshd_gnutls_audit_func);
 
-	tlshd_log_debug("System config file: %s", gnutls_get_system_config_file());
+#ifdef HAVE_GNUTLS_GET_SYSTEM_CONFIG_FILE
+	tlshd_log_debug("System config file: %s",
+			gnutls_get_system_config_file());
+#endif
 
 	switch (parms->auth_mode) {
 	case HANDSHAKE_AUTH_X509:
